﻿/*:
 * @target MZ
 * @plugindesc Plugin where the player is surrounded by a flame aura
 * @author Malomi
 * @url https://gendaigoyaku.com/
 *
 * @param Image X Offset
 * @text X Offset
 * @desc X-axis offset from the player's position
 * @default 0
 *
 * @param Image Y Offset
 * @text Y Offset
 * @desc Y-axis offset from the player's position
 * @default 24
 *
 * @param Scale Factor
 * @text Magnification ratio
 * @desc Magnification ratio of the displayed flame
 * @default 2.0
 * @type number
 * @decimals 2
 * 
 * @param Show Flame Initially
 * @text Initial display of the flame
 * @desc Whether to display the flame at the start of the game
 * @default true
 * @type boolean
 *
 * @requiredAssets img/characters/!Other2
 * 
 * @command showFlame
 * @text Display flame
 * @desc We will display the player's flame aura.
 * 
 * @command hideFlame
 * @text Hide flame
 * @desc We will hide the player's flame aura.
 * 
 * @help
 * ----------------------------------------------------------------------------
 * FlamingAura.js
 * ----------------------------------------------------------------------------
 * This plugin displays flames at the player's position.
 * You can freely adjust the size of the flames.
 * 
 * If changing the size of the flames causes a positional shift,
 * you can also adjust the position using offsets.
 * 
 * Plugin command:
 * - "showFlame" -> Display the flame aura.
 * - "hideFlame" -> Hide the flame aura.
 * 
 * ----------------------------------------------------------------------------
 * License:
 * This is the MIT License. 
 * It can be used for both commercial and non-commercial purposes.
 * ----------------------------------------------------------------------------
 */

/*:ja
 * @target MZ
 * @plugindesc プレイヤーが炎のオーラをまとうプラグイン
 * @author まろみ
 * @url https://gendaigoyaku.com/
 *
 * @param Image X Offset
 * @text Xオフセット
 * @desc プレイヤーの位置からのX軸のオフセット
 * @default 0
 *
 * @param Image Y Offset
 * @text Yオフセット
 * @desc プレイヤーの位置からのY軸のオフセット
 * @default 24
 *
 * @param Scale Factor
 * @text 拡大倍率
 * @desc 表示する炎の拡大倍率
 * @default 2.0
 * @type number
 * @decimals 2
 * 
 * @param Show Flame Initially
 * @text 炎の初期表示
 * @desc ゲーム開始時に炎を表示するかどうか
 * @default true
 * @type boolean
 *
 * @requiredAssets img/characters/!Other2
 * 
 * @command showFlame
 * @text 炎を表示
 * @desc プレイヤーの炎のオーラを表示します。
 * 
 * @command hideFlame
 * @text 炎を非表示
 * @desc プレイヤーの炎のオーラを非表示にします。
 * 
 * @help
 * ----------------------------------------------------------------------------
 * FlamingAura.js
 * ----------------------------------------------------------------------------
 * このプラグインは、プレイヤーの位置に炎を表示します。
 * 炎の大きさは自由に設定できます。
 * 
 * 炎の大きさを変えて、位置がずれた場合は
 * オフセットで位置も設定できます。
 * 
 * プラグインコマンド:
 * - "炎を表示"   -> 炎のオーラを表示します。
 * - "炎を非表示" -> 炎のオーラを非表示にします。
 * 
 * ----------------------------------------------------------------------------
 * 利用規約:
 * 商用・非商用問わず利用可能です。
 * ----------------------------------------------------------------------------
 */

(() => {
    'use strict';

    const parameters = PluginManager.parameters('FlamingAura');
    const frameIndex = 10;
    const xOffset = Number(parameters['Image X Offset'] || 0);
    const yOffset = Number(parameters['Image Y Offset'] || 24);
    const animationSpeed = 12;
    const scaleFactor = Number(parameters['Scale Factor'] || 2.0);
    let isFlameVisible = String(parameters['Show Flame Initially']).toLowerCase() === 'true';

    class OverlayFlameSprite extends Sprite {
        constructor() {
            super();
            this.bitmap = ImageManager.loadCharacter('!Other2');
            this.frameIndex = frameIndex;
            this.anchor.x = 0.5;
            this.anchor.y = 0.5;
            this.opacity = 128;
            this.animationFrame = 0;
            this.scale.x = scaleFactor;
            this.scale.y = scaleFactor;
            this.visible = isFlameVisible;
        }

        updatePosition(characterSprite) {
            if (!this.bitmap.isReady() || !characterSprite) {
                return;
            }

            if (isFlameVisible) {
                const cw = this.bitmap.width / 12;
                const ch = this.bitmap.height / 8;

                const animationFrames = [9, 10, 11, 10];
                this.frameIndex = animationFrames[Math.floor(this.animationFrame / animationSpeed) % animationFrames.length];
                this.animationFrame++;

                const sx = (this.frameIndex % 12) * cw;
                const sy = Math.floor(this.frameIndex / 12) * ch;

                this.setFrame(sx, sy, cw, ch);
                this.x = xOffset;
                this.y = -ch + yOffset;
                if (!this.visible) {
                    this.visible = true;
                }
            } else {
                if (this.visible) {
                    this.visible = false;
                }
            }
        }
    }

    const _Scene_Map_createSpriteset = Scene_Map.prototype.createSpriteset;
    Scene_Map.prototype.createSpriteset = function () {
        _Scene_Map_createSpriteset.call(this);
        const playerSprite = this._spriteset._characterSprites.find(sprite => sprite._character === $gamePlayer);
        if (playerSprite) {
            this._overlayCharacterSprite = new OverlayFlameSprite();
            this._overlayCharacterSprite.z = playerSprite.z + 1;
            playerSprite.addChild(this._overlayCharacterSprite);
        } else {
            console.warn('Player sprite not found.');
        }
    };

    const _Scene_Map_update = Scene_Map.prototype.update;
    Scene_Map.prototype.update = function () {
        _Scene_Map_update.call(this);
        if (this._overlayCharacterSprite) {
            const playerSprite = this._spriteset._characterSprites.find(sprite => sprite._character === $gamePlayer);
            if (playerSprite) {
                this._overlayCharacterSprite.updatePosition(playerSprite);
            } else {
                console.warn('Player sprite not found during update.');
            }
        }
    };

    PluginManager.registerCommand('FlamingAura', 'showFlame', () => {
        isFlameVisible = true;
        const scene = SceneManager._scene;
        if (scene instanceof Scene_Map && scene._overlayCharacterSprite) {
            scene._overlayCharacterSprite.visible = true;
        }
    });

    PluginManager.registerCommand('FlamingAura', 'hideFlame', () => {
        isFlameVisible = false;
        const scene = SceneManager._scene;
        if (scene instanceof Scene_Map && scene._overlayCharacterSprite) {
            scene._overlayCharacterSprite.visible = false;
        }
    });
})();
